require 'restclient'
require 'json'
require 'date'
require 'digest/sha1'
require 'openssl'
require 'base64'
require 'exceptions/campoInvalido'

class Soullab

  $DEBUG = false

  $base_url = "http://www.soullab.com.br/SoullabWebService/service/"

  def initialize(company, keyPath, keyPass, language)
  	@company = company
	@keyPath = keyPath
	@keyPass = keyPass
	@language = language
  end
	
  def integrarEscola(codigo, nomeEscola, cep, cidade, uf, bairro, rua, numero, complemento, telefone, celular, nomeGestor, emailGestor, login)

	puts "Generating json..." if $DEBUG

	escola = {
		endereco: {
			cep: cep,
			cidade: cidade,
			uf: uf,
			bairro: bairro,
			rua: rua,
			numero: numero,
			complemento: complemento
		},
		gestor: {
			nome: nomeGestor,
			email: emailGestor,
			login: login
			
		},
		codigo: codigo,
		nomeEscola: nomeEscola,		
		telefone: telefone,
		celular: celular,
		nomeContato: nomeGestor,
		emailContato: emailGestor
	}
	
	timestamp = DateTime.now.strftime('%Q')
	
	mensagem = {	
		jsonObject: escola.to_json,
		timestamp: timestamp,
		empresa: @company
	}		
	
	puts "Json generated." if $DEBUG
	puts "Generating SHA1 hash using base64..." if $DEBUG
	
	raw_content = mensagem.to_json
	
	puts raw_content
	
	assinatura = Digest::SHA1.base64digest raw_content
	
	puts "THE HASH IS: #{assinatura}" if $DEBUG
	
	puts "SHA1 hash generated." if $DEBUG
	puts "Signing hash..." if $DEBUG
	
	keyfile = File.read @keyPath
	private_key = OpenSSL::PKey::RSA.new(keyfile, @keyPass)
	
	assinatura = private_key.private_encrypt assinatura
	
	puts "Hash signed." if $DEBUG
	
	mensagem[:assinatura] = Base64.encode64(assinatura)
	
	mensagem[:idioma] = @language
	
	puts "JSON: #{mensagem.to_json}" if $DEBUG	
	
	url = $base_url + "escola/cadastrarEscola"

	response = RestClient.post(
	  url,
	  mensagem.to_json,
	  :content_type => :json,
	  :accept => :json
	)
	
	return JSON.parse(response.body)
  end
  
  def integrarProfessor(codigoEscola, login, nome, rua, numero, complemento, bairro, cidade, cep, uf, telefone, celular, email)

	puts "Generating json..." if $DEBUG

	professor = {
		endereco: {
			cep: cep,
			cidade: cidade,
			uf: uf,
			bairro: bairro,
			rua: rua,
			numero: numero,
			complemento: complemento
		},
		nome: nome,
		email: email,
		login: login,
		codigoEscola: codigoEscola,
		telefone: telefone,
		celular: celular
	}
	
	timestamp = DateTime.now.strftime('%Q')
	
	mensagem = {	
		jsonObject: professor.to_json,
		timestamp: timestamp,
		empresa: @company
	}		
	
	puts "Json generated." if $DEBUG
	puts "Generating SHA1 hash using base64..." if $DEBUG
	
	raw_content = mensagem.to_json
	
	puts raw_content
	
	assinatura = Digest::SHA1.base64digest raw_content
	
	puts "THE HASH IS: #{assinatura}" if $DEBUG
	
	puts "SHA1 hash generated." if $DEBUG
	puts "Signing hash..." if $DEBUG
	
	keyfile = File.read @keyPath
	private_key = OpenSSL::PKey::RSA.new(keyfile, @keyPass)
	
	assinatura = private_key.private_encrypt assinatura
	
	puts "Hash signed." if $DEBUG
	
	mensagem[:assinatura] = Base64.encode64(assinatura)
	
	mensagem[:idioma] = @language
	
	puts "JSON: #{mensagem.to_json}" if $DEBUG	
	
	url = $base_url + "professor/cadastrarProfessor"

	response = RestClient.post(
	  url,
	  mensagem.to_json,
	  :content_type => :json,
	  :accept => :json
	)
	
	#wsreturn = JSON.parse(response.body)	
	#begin
	#raise CampoInvalido.new("login","Este parâmetro deve ser alfanumérico") if wsreturn['message'] == "23"
	#rescue CampoInvalido => e	
	#puts "campo: #{e.campo}"
	#puts "motivo: #{e.motivo}"
	#end
		
	return JSON.parse(response.body)
  end
  
  def integrarTurma(codigoEscola, codigo, professor, dataInicio, dataFim, avisos)
  	
	puts "Generating json..." if $DEBUG

	turma = {		
		codigoEscola: codigoEscola,
		codigo: codigo,
		professor: professor,
		dataInicio: dataInicio,
		dataFim: dataFim,
		avisos: avisos
	}

	timestamp = DateTime.now.strftime('%Q')

	mensagem = {	
		jsonObject: turma.to_json,
		timestamp: timestamp,
		empresa: @company
	}		

	puts "Json generated." if $DEBUG
	puts "Generating SHA1 hash using base64..." if $DEBUG

	raw_content = mensagem.to_json

	puts raw_content

	assinatura = Digest::SHA1.base64digest raw_content

	puts "THE HASH IS: #{assinatura}" if $DEBUG

	puts "SHA1 hash generated." if $DEBUG
	puts "Signing hash..." if $DEBUG

	keyfile = File.read @keyPath
	private_key = OpenSSL::PKey::RSA.new(keyfile, @keyPass)

	assinatura = private_key.private_encrypt assinatura

	puts "Hash signed." if $DEBUG

	mensagem[:assinatura] = Base64.encode64(assinatura)

	mensagem[:idioma] = @language

	puts "JSON: #{mensagem.to_json}" if $DEBUG

	url = $base_url + "turma/cadastrarTurma"

	response = RestClient.post(
	  url,
	  mensagem.to_json,
	  :content_type => :json,
	  :accept => :json
	)

	return JSON.parse(response.body)
  
  end
  
  def integrarAluno(codigoEscola, login, nome, rua, numero, complemento, bairro, cidade, cep, uf, telefone, celular, email)

	puts "Generating json..." if $DEBUG

	aluno = {
		endereco: {
			cep: cep,
			cidade: cidade,
			uf: uf,
			bairro: bairro,
			rua: rua,
			numero: numero,
			complemento: complemento
		},
		nome: nome,
		email: email,
		login: login,
		codigoEscola: codigoEscola,
		telefone: telefone,
		celular: celular
	}
	
	timestamp = DateTime.now.strftime('%Q')
	
	mensagem = {	
		jsonObject: aluno.to_json,
		timestamp: timestamp,
		empresa: @company
	}		
	
	puts "Json generated." if $DEBUG
	puts "Generating SHA1 hash using base64..." if $DEBUG
	
	raw_content = mensagem.to_json
	
	puts raw_content
	
	assinatura = Digest::SHA1.base64digest raw_content
	
	puts "THE HASH IS: #{assinatura}" if $DEBUG
	
	puts "SHA1 hash generated." if $DEBUG
	puts "Signing hash..." if $DEBUG
	
	keyfile = File.read @keyPath
	private_key = OpenSSL::PKey::RSA.new(keyfile, @keyPass)
	
	assinatura = private_key.private_encrypt assinatura
	
	puts "Hash signed." if $DEBUG
	
	mensagem[:assinatura] = Base64.encode64(assinatura)
	
	mensagem[:idioma] = @language
	
	puts "JSON: #{mensagem.to_json}" if $DEBUG	
	
	url = $base_url + "aluno/cadastrarAluno"

	response = RestClient.post(
	  url,
	  mensagem.to_json,
	  :content_type => :json,
	  :accept => :json
	)
	
	return JSON.parse(response.body)
  end
  
  def integrarHorarios(codigoEscola, codigoTurma, horarios)

	puts "Generating json..." if $DEBUG

	horarios = {
		codigoEscola: codigoEscola,
		codigoTurma: codigoTurma,
		horarios: horarios
	}
	
	timestamp = DateTime.now.strftime('%Q')
	
	mensagem = {	
		jsonObject: horarios.to_json,
		timestamp: timestamp,
		empresa: @company
	}		
	
	puts "Json generated." if $DEBUG
	puts "Generating SHA1 hash using base64..." if $DEBUG
	
	raw_content = mensagem.to_json
	
	puts raw_content
	
	assinatura = Digest::SHA1.base64digest raw_content
	
	puts "THE HASH IS: #{assinatura}" if $DEBUG
	
	puts "SHA1 hash generated." if $DEBUG
	puts "Signing hash..." if $DEBUG
	
	keyfile = File.read @keyPath
	private_key = OpenSSL::PKey::RSA.new(keyfile, @keyPass)
	
	assinatura = private_key.private_encrypt assinatura
	
	puts "Hash signed." if $DEBUG
	
	mensagem[:assinatura] = Base64.encode64(assinatura)
	
	mensagem[:idioma] = @language
	
	puts "JSON: #{mensagem.to_json}" if $DEBUG	
	
	url = $base_url + "horario/cadastrarHorario"

	response = RestClient.post(
	  url,
	  mensagem.to_json,
	  :content_type => :json,
	  :accept => :json
	)
	
	return JSON.parse(response.body)
  end
  
  def integrarEnturmamento(codigoEscola, codigoTurma, matriculas)

	puts "Generating json..." if $DEBUG

	enturmamento = {
		codigoEscola: codigoEscola,
		codigoTurma: codigoTurma,
		matriculas: matriculas
	}
	
	timestamp = DateTime.now.strftime('%Q')
	
	mensagem = {	
		jsonObject: enturmamento.to_json,
		timestamp: timestamp,
		empresa: @company
	}		
	
	puts "Json generated." if $DEBUG
	puts "Generating SHA1 hash using base64..." if $DEBUG
	
	raw_content = mensagem.to_json
	
	puts raw_content
	
	assinatura = Digest::SHA1.base64digest raw_content
	
	puts "THE HASH IS: #{assinatura}" if $DEBUG
	
	puts "SHA1 hash generated." if $DEBUG
	puts "Signing hash..." if $DEBUG
	
	keyfile = File.read @keyPath
	private_key = OpenSSL::PKey::RSA.new(keyfile, @keyPass)
	
	assinatura = private_key.private_encrypt assinatura
	
	puts "Hash signed." if $DEBUG
	
	mensagem[:assinatura] = Base64.encode64(assinatura)
	
	mensagem[:idioma] = @language
	
	puts "JSON: #{mensagem.to_json}" if $DEBUG	
	
	url = $base_url + "enturmamento/cadastrarEnturmamento"

	response = RestClient.post(
	  url,
	  mensagem.to_json,
	  :content_type => :json,
	  :accept => :json
	)
	
	return JSON.parse(response.body)
  end
  
end
